/*
                     This file is part of the OPIE Project
                             Copyright (c)  2002 Trolltech AS <info@trolltech.com>
               =.            Copyright (c)  2002 Dan Williams <williamsdr@acm.org>               
      .=l.            Copyright (c)  2002 Robert Griebl <sandman@handhelds.org>
     .>+-=
_;:,   .>  :=|.         This file is free software; you can
.> <`_,  > .  <=          redistribute it and/or modify it under
:`=1 )Y*s>-.--  :           the terms of the GNU General Public
.="- .-=="i,   .._         License as published by the Free Software
- .  .-<_>   .<>         Foundation; either version 2 of the License,
  ._= =}    :          or (at your option) any later version.
  .%`+i>    _;_.
  .i_,=:_.   -<s.       This file is distributed in the hope that
  + . -:.    =       it will be useful, but WITHOUT ANY WARRANTY;
  : ..  .:,   . . .    without even the implied warranty of
  =_    +   =;=|`    MERCHANTABILITY or FITNESS FOR A
 _.=:.    :  :=>`:     PARTICULAR PURPOSE. See the GNU General
..}^=.=    =    ;      Public License for more details.
++=  -.   .`   .:       
:   = ...= . :.=-        You should have received a copy of the GNU
-.  .:....=;==+<;          General Public License along with this file;
 -_. . .  )=. =           see the file COPYING. If not, write to the
  --    :-=`           Free Software Foundation, Inc.,
                             59 Temple Place - Suite 330,
                             Boston, MA 02111-1307, USA.

*/

#include <dlfcn.h>

#include <qbuttongroup.h>
#include <qwidgetstack.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdialog.h>
#include <qdir.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#if QT_VERSION >= 300
#include <qstylefactory.h>
#endif
#include <qtoolbutton.h>
#include <qwindowsstyle.h>
#include <qlistview.h>
#include <qheader.h>
#include <qvbox.h>

#include <qpe/config.h>
#include <qpe/global.h>
#include <qpe/resource.h>
#include <qpe/qpeapplication.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/qpestyle.h>
#include <qpe/lightstyle.h>
#include <qpe/qlibrary.h>
#include <qpe/styleinterface.h>

#include <opie/ocolorbutton.h>
#include <opie/ofontselector.h>
//#include <opie/odevice.h>
#include <opie/ofiledialog.h>
#include <opie/otabwidget.h>

#include "appearance.h"
#include "editScheme.h"
#include "stylelistitem.h"
//#include "decolistitem.h"
#include "colorlistitem.h"
#include "exceptlistitem.h"
#include "sample.h"

namespace Opie { }

using namespace Opie;


class MyListView : public QListView {
public:
	MyListView ( QWidget *parent = 0, const char *name = 0 )
		: QListView ( parent, name )
	{ }
	
	virtual QSize sizeHint ( ) const
	{
		QSize s = QListView::sizeHint ( );
		s. setHeight ( 30 );
	
		return s;
	}
};




QWidget *Appearance::createStyleTab ( QWidget *parent, Config &cfg )
{
	QWidget* tab = new QWidget( parent, "StyleTab" );
	QVBoxLayout* vertLayout = new QVBoxLayout( tab, 3, 3 );

	m_style_list = new QListBox( tab, "m_style_list" );
	vertLayout->addWidget( m_style_list );

	m_style_settings = new QPushButton ( tr( "Settings..." ), tab );
	connect ( m_style_settings, SIGNAL( clicked ( )), this, SLOT( styleSettingsClicked ( )));
	vertLayout-> addWidget ( m_style_settings );

	QString s = cfg. readEntry ( "Style", "Light" );
	

#if QT_VERSION >= 300
	m_style_list->insertStringList(QStyleFactory::styles());
#else
	m_style_list-> insertItem ( new StyleListItem ( "Windows", new QWindowsStyle ( )));
	m_style_list-> insertItem ( new StyleListItem ( "Light", new LightStyle ( )));
	m_style_list-> insertItem ( new StyleListItem ( "QPE", new QPEStyle ( )));
#endif
	{
		QString path = QPEApplication::qpeDir ( ) + "/plugins/styles/";
		QStringList sl = QDir ( path, "lib*.so" ). entryList ( );

		for ( QStringList::Iterator it = sl. begin ( ); it != sl. end ( ); ++it ) {
			if ( *it == "libzstyle.so" )
				continue;		
		
			QLibrary *lib = new QLibrary ( path + "/" + *it );
			StyleInterface *iface;

			if (( lib-> queryInterface ( IID_Style, (QUnknownInterface **) &iface ) == QS_OK ) && iface ) {
				StyleListItem *slit = new StyleListItem ( lib, iface );
				m_style_list-> insertItem ( slit );
				
				if ( slit-> key ( ) == s )
					m_style_list-> setCurrentItem ( slit );
			}
			else
				delete lib;
		}
	}

	m_original_style = m_style_list-> currentItem ( );
	styleClicked ( m_original_style );

	connect( m_style_list, SIGNAL( highlighted( int ) ), this, SLOT( styleClicked( int ) ) );

	return tab;
}

#if 0
QWidget *Appearance::createDecoTab ( QWidget *parent, Config &cfg )
{
	QWidget* tab = new QWidget( parent, "DecoTab" );
	QVBoxLayout* vertLayout = new QVBoxLayout( tab, 3, 3 );

	m_deco_list = new QListBox( tab, "m_deco_list" );
	vertLayout->addWidget( m_deco_list );

	QString s = cfg. readEntry ( "Decoration" );

	m_deco_list-> insertItem ( new DecoListItem ( "QPE" ));

	{
		QString path = QPEApplication::qpeDir() + "/plugins/decorations/";
		QStringList sl = QDir ( path, "lib*.so" ). entryList ( );

		for ( QStringList::Iterator it = sl. begin ( ); it != sl. end ( ); ++it ) {
			QLibrary *lib = new QLibrary ( path + "/" + *it );
			WindowDecorationInterface *iface;

			if ( lib-> queryInterface ( IID_WindowDecoration, (QUnknownInterface **) &iface ) == QS_OK ) {
				DecoListItem *dlit = new DecoListItem ( lib, iface );				
				m_deco_list-> insertItem ( dlit );
				
				if ( dlit-> key ( ) == s )
					m_deco_list-> setCurrentItem ( dlit );
			}
			else
				delete lib;
		}
	}

	m_original_deco = m_deco_list-> currentItem ( );
	if ( m_deco_list-> currentItem  ( ) < 0 )
		m_deco_list-> setCurrentItem ( 0 );
	decoClicked ( m_original_deco );

	connect( m_deco_list, SIGNAL( highlighted( int ) ), this, SLOT( decoClicked( int ) ) );

	return tab;
}
#endif

QWidget *Appearance::createFontTab ( QWidget *parent, Config &cfg )
{
	QString familyStr = cfg. readEntry ( "FontFamily", "Helvetica" );
	QString styleStr = cfg. readEntry ( "FontStyle", "Regular" );
	int size = cfg. readNumEntry ( "FontSize", 10 );

	m_fontselect = new OFontSelector ( false, parent, "FontTab" );	
	m_fontselect-> setSelectedFont ( familyStr, styleStr, size );
	
	connect( m_fontselect, SIGNAL( fontSelected ( const QFont & )),
	         this, SLOT( fontClicked ( const QFont & )));

	return m_fontselect; 
}

QWidget *Appearance::createColorTab ( QWidget *parent, Config &cfg )
{
	QWidget *tab = new QWidget( parent, "ColorTab" );
	QGridLayout *gridLayout = new QGridLayout( tab, 0, 0, 3, 3 );
	gridLayout->setRowStretch ( 3, 10 );

	m_color_list = new QListBox ( tab );
	gridLayout->addMultiCellWidget ( m_color_list, 0, 3, 0, 0 );
	connect( m_color_list, SIGNAL( highlighted( int ) ), this, SLOT( colorClicked( int ) ) );

	m_color_list-> insertItem ( new ColorListItem ( tr( "Current scheme" ), cfg ));

	QString path = QPEApplication::qpeDir ( ) + "/etc/colors/";
	QStringList sl = QDir ( path ). entryList ( "*.scheme" );
	
	for ( QStringList::Iterator it = sl. begin ( ); it != sl. end ( ); ++it ) {
		QString name = (*it). left ((*it). find ( ".scheme" ));
		Config config ( path + *it, Config::File );
		config. setGroup ( "Colors" );
		
		m_color_list-> insertItem ( new ColorListItem ( name, config ));
	}
	
	m_color_list-> setCurrentItem ( 0 );

	QPushButton* tempButton = new QPushButton( tab, "editSchemeButton" );
	tempButton->setText( tr( "Edit..." ) );
	connect( tempButton, SIGNAL( clicked() ), this, SLOT( editSchemeClicked() ) );
	gridLayout->addWidget( tempButton, 0, 1 );

	tempButton = new QPushButton( tab, "deleteSchemeButton" );
	tempButton->setText( tr( "Delete" ) );
	connect( tempButton, SIGNAL( clicked() ), this, SLOT( deleteSchemeClicked() ) );
	gridLayout->addWidget( tempButton, 1, 1 );

	tempButton = new QPushButton( tab, "saveSchemeButton" );
	tempButton->setText( tr( "Save" ) );
	connect( tempButton, SIGNAL( clicked() ), this, SLOT( saveSchemeClicked() ) );
	gridLayout->addWidget( tempButton, 2, 1 );

	return tab;
}

QWidget *Appearance::createAdvancedTab ( QWidget *parent, Config &cfg )
{
	QWidget *tab = new QWidget ( parent );
	QVBoxLayout *vertLayout = new QVBoxLayout( tab, 3, 3 );

	QGridLayout *lay = new QGridLayout ( vertLayout, 0, 0, 3, 0 );

	m_force = new QCheckBox ( tr( "Force styling for all applications." ), tab );
	m_force-> setChecked ( cfg. readBoolEntry ( "ForceStyle" ));
	lay-> addMultiCellWidget ( m_force, 0, 0, 0, 1 );	   

	QLabel *l = new QLabel ( tab );
	l-> setText ( QString ( "<p>%1</p>" ). arg ( tr( "Disable styling for these applications ( <b>*</b> can be used as a wildcard):" )));
	lay-> addMultiCellWidget ( l, 1, 1, 0, 1 );
	
	m_except = new MyListView ( tab );
	m_except-> addColumn ( Resource::loadIconSet ( "zstyle/style.png" ), "", 24 );
	m_except-> addColumn ( Resource::loadIconSet ( "zstyle/font.png" ), "", 24 );
	m_except-> addColumn ( Resource::loadIconSet ( "zstyle/deco.png" ), "", 0  );
	m_except-> addColumn ( tr( "Binary file(s)" ));
	m_except-> setColumnAlignment ( 0, AlignCenter );
	m_except-> setColumnAlignment ( 1, AlignCenter );
	m_except-> setColumnAlignment ( 2, AlignCenter );
	m_except-> setAllColumnsShowFocus ( true );
	m_except-> setMinimumHeight ( 30 );
	m_except-> header ( )-> setClickEnabled ( false );
	m_except-> header ( )-> setResizeEnabled ( false );
	m_except-> header ( )-> setMovingEnabled ( false );	
	lay-> addMultiCellWidget ( m_except, 2, 6, 0, 0 );
	
	connect ( m_except, SIGNAL( clicked ( QListViewItem *, const QPoint &, int )), this, SLOT( clickedExcept ( QListViewItem *, const QPoint &, int )));
	
	QToolButton *tb = new QToolButton ( tab );
	tb-> setIconSet ( Resource::loadIconSet ( "zstyle/add" ));
	tb-> setFocusPolicy ( QWidget::StrongFocus );
	lay-> addWidget ( tb, 2, 1 ); 
	connect ( tb, SIGNAL( clicked ( )), this, SLOT( addExcept ( )));

	tb = new QToolButton ( tab );
	tb-> setIconSet ( Resource::loadIconSet ( "editdelete" ));
	tb-> setFocusPolicy ( QWidget::StrongFocus );
	lay-> addWidget ( tb, 3, 1 ); 
	connect ( tb, SIGNAL( clicked ( )), this, SLOT( delExcept ( )));
	
	tb = new QToolButton ( tab );
	tb-> setIconSet ( Resource::loadIconSet ( "up" ));
	tb-> setFocusPolicy ( QWidget::StrongFocus );
	lay-> addWidget ( tb, 4, 1 ); 
	connect ( tb, SIGNAL( clicked ( )), this, SLOT( upExcept ( )));
	
	tb = new QToolButton ( tab );
	tb-> setIconSet ( Resource::loadIconSet ( "down" ));
	tb-> setFocusPolicy ( QWidget::StrongFocus );
	lay-> addWidget ( tb, 5, 1 ); 
	connect ( tb, SIGNAL( clicked ( )), this, SLOT( downExcept ( )));
	
	lay-> setRowStretch ( 6, 10 );
	lay-> setColStretch ( 0, 10 );

	QStringList sl = cfg. readListEntry ( "NoStyle", ';' );
	QListViewItem *lvit = 0;
	for ( QStringList::Iterator it = sl. begin ( ); it != sl. end ( ); ++it ) {
		int fl = ( *it ). left ( 1 ). toInt ( 0, 32 );
	
		lvit = new ExceptListItem ( m_except, lvit, ( *it ). mid ( 1 ), fl & 0x01, fl & 0x02, fl & 0x04 );
	}

#if 0
	vertLayout-> addSpacing ( 3 );	
	QFrame *f = new QFrame ( tab );
	f-> setFrameStyle ( QFrame::HLine | QFrame::Sunken );
	vertLayout-> addWidget ( f ); 
	vertLayout-> addSpacing ( 3 );   
		
		
	QGridLayout* gridLayout = new QGridLayout ( vertLayout, 0, 0, 3, 0 );

	int style = cfg. readNumEntry ( "TabStyle", 2 ) - 1;
	bool tabtop = ( cfg. readEntry ( "TabPosition", "Top" ) == "Top" );

	QLabel* label = new QLabel( tr( "Tab style:" ), tab );
	gridLayout-> addWidget ( label, 0, 0 );
	QButtonGroup* btngrp = new QButtonGroup( tab, "buttongroup" );
	btngrp-> hide ( );
	btngrp-> setExclusive ( true );

	m_tabstyle_list = new QComboBox ( false, tab, "tabstyle" );
	m_tabstyle_list-> insertItem ( tr( "Tabs" ));
	m_tabstyle_list-> insertItem ( tr( "Tabs w/icons" ));
	m_tabstyle_list-> insertItem ( tr( "Drop down list" ));
	m_tabstyle_list-> insertItem ( tr( "Drop down list w/icons" ));
	m_tabstyle_list-> setCurrentItem ( style );
	gridLayout-> addMultiCellWidget ( m_tabstyle_list, 0, 0, 1, 2 );

	m_tabstyle_top = new QRadioButton( tr( "Top" ), tab, "tabpostop" );
	btngrp-> insert ( m_tabstyle_top );
	gridLayout-> addWidget( m_tabstyle_top, 1, 1 );
	
	m_tabstyle_bottom = new QRadioButton( tr( "Bottom" ), tab, "tabposbottom" );
	btngrp-> insert ( m_tabstyle_bottom );
	gridLayout-> addWidget( m_tabstyle_bottom, 1, 2 );

	m_tabstyle_top-> setChecked ( tabtop );
	m_tabstyle_bottom-> setChecked ( !tabtop );

	m_original_tabstyle = style;
	m_original_tabpos = tabtop;
#endif

	return tab;
}


Appearance::Appearance( QWidget* parent,  const char* name, WFlags )
	: QDialog ( parent, name, true )
{
	setCaption( tr( "Appearance" ) );


	int (*adr) ( ) = (int (*) ()) ::dlsym ( 0, "__zstyle_version__" );
	if ( adr )
		m_version = (*adr) ( );
	else
		m_version = 0;

	qDebug ( "m_version = %d (adr=%p)", m_version, adr );

	Config config( "qpe" );
	config.setGroup( "Appearance" );

	QVBoxLayout *top = new QVBoxLayout ( this, 3, 3 );

	m_color_list = 0;

	m_sample_stack = new QWidgetStack ( this );
	m_sample  = new SampleWindow ( m_sample_stack );
	m_sample_stack-> addWidget ( m_sample, 0 );
	m_sample2 = new SampleView ( m_sample_stack );
	m_sample_stack-> addWidget ( m_sample2, 1 );
	
	
	OTabWidget* tw = new OTabWidget ( this, "tabwidget", OTabWidget::Global, OTabWidget::Bottom );
	QWidget *styletab;
	
 	tw-> addTab ( styletab = createStyleTab ( tw, config ), "zstyle/style.png", tr( "Style" ));
	tw-> addTab ( createFontTab ( tw, config ), "zstyle/font.png", tr( "Font" ));
	tw-> addTab ( createColorTab ( tw, config ), "zstyle/color.png", tr( "Colors" ) );
	tw-> addTab ( m_launchtab = createLauncherTab ( tw, config ), "zstyle/launcher.png", tr( "Launcher" ) );
	tw-> addTab ( m_advtab = createAdvancedTab ( tw, config ), "zstyle/advanced.png", tr( "Advanced" ) );

	top-> addWidget ( tw, 10 );	
	top-> addWidget ( m_sample_stack, 1 );

	tw-> setCurrentTab ( styletab );
	connect ( tw, SIGNAL( currentChanged ( QWidget * )), this, SLOT( tabChanged ( QWidget * )));
		
	m_style_changed = m_font_changed = m_color_changed = m_deco_changed = false;
}

Appearance::~Appearance()
{
}

void Appearance::tabChanged ( QWidget *w )
{
	if ( w == m_advtab ) {
		m_sample_stack-> hide ( );
		updateGeometry ( ); // shouldn't be necessary ...
	}
	else {
		if ( w == m_launchtab )
			m_sample_stack-> raiseWidget ( 1 );
		else
			m_sample_stack-> raiseWidget ( 0 );

		m_sample_stack-> show ( );
	}
}

void Appearance::accept ( )
{
#if 0
	bool newtabpos = m_tabstyle_top-> isChecked ( );
	int newtabstyle = m_tabstyle_list-> currentItem ( );
#endif

	Config config ( "qpe" );
	config. setGroup ( "Appearance" );

	if ( m_style_changed ) { 
		StyleListItem *item = (StyleListItem *) m_style_list-> item ( m_style_list-> currentItem ( ));
		if ( item )
			config.writeEntry( "Style", item-> key ( ));
	}
#if 0	
	if ( m_deco_changed ) { 
		DecoListItem *item = (DecoListItem *) m_deco_list-> item ( m_deco_list-> currentItem ( ));
		if ( item )
			config.writeEntry( "Decoration", item-> key ( ));
	}	

	if (( newtabstyle != m_original_tabstyle ) || ( newtabpos != m_original_tabpos )) {
		config. writeEntry ( "TabStyle", newtabstyle + 1 );
		config. writeEntry ( "TabPosition", newtabpos ? "Top" : "Bottom" );
	}
#endif
	if ( m_font_changed ) {
		config. writeEntry ( "FontFamily", m_fontselect-> fontFamily ( ));
		config. writeEntry ( "FontStyle", m_fontselect-> fontStyle ( ));
		config. writeEntry ( "FontSize", m_fontselect-> fontSize ( ));
	}


	if ( m_color_changed )
	{
		ColorListItem *item = (ColorListItem *) m_color_list-> item ( m_color_list-> currentItem ( ));
 
 		if ( item )   
			item-> save ( config );
	}
	
	
	m_except-> setFocus ( ); // if the focus was on the embedded line-edit, we have to move it away first, so the contents are updated

	QStringList sl;
	for ( ExceptListItem *it = (ExceptListItem *) m_except-> firstChild ( ); it; it = (ExceptListItem *) it-> nextSibling ( )) {
		int fl = 0;
		fl |= ( it-> noStyle ( ) ? 0x01 : 0 );
		fl |= ( it-> noFont ( ) ? 0x02 : 0 );
		fl |= ( it-> noDeco ( ) ? 0x04 : 0 );
		sl << ( QString::number ( fl, 32 ) + it-> pattern ( ));
	}
	config. writeEntry ( "NoStyle", sl, ';' );
	config. writeEntry ( "ForceStyle", m_force-> isChecked ( ));

	config. write ( ); // need to flush the config info first


	Config cfg ( "zstyle" );
	cfg. setGroup ( "Launcher" );

	if ( m_bgtype ) {
	QString s;
	switch  ( m_bgtype-> id ( m_bgtype-> selected ( ))) {
		case Image     : s = "Image"; break;
		case SolidColor: s = "SolidColor"; break;
		case Ruled     : 
		default        : s = "Ruled"; break;
	}
	cfg. writeEntry ( "BackgroundType", s );
	}	
		
	cfg. writeEntry ( "BackgroundImage", m_bgimage );
	if ( m_solidcolor )
		cfg. writeEntry ( "BackgroundColor", m_solidcolor-> color ( ). name ( ));
	
	cfg. writeEntry ( "TextColor", m_iconcolor_use-> isChecked ( ) ? m_iconcolor-> color ( ). name ( ) : QString::null );
	cfg. writeEntry ( "DocView", m_smallicons-> isChecked ( ) ? "List" : "Icon" );

	QStringList isl;
	if ( m_iconfont_use-> isChecked ( )) {	
		QFont f = m_iconfont_select-> selectedFont ( );
	
		isl << f. family ( ) 
		   << QString::number ( f. pointSize ( ))
		   << QString::number ( f. weight ( ))
		   << ( f. italic ( ) ? "1" : "0" );
	}
	cfg. writeEntry ( "Font", isl, ',' );

	cfg. write ( );
	
	Global::applyStyle ( );

	if ( QMessageBox::warning ( this, tr( "Restart" ), tr( "<p>Do you want to restart Qtopia now?<br><br>(This is normally only needed if you changed the launcher settings)</p>" ),  tr( "Yes" ), tr( "No" ), 0, 0, 1 ) == 0 ) {
		QCopEnvelope e( "QPE/System", "restart()" );
	}
	
	QDialog::accept ( );
}

void Appearance::done ( int r )
{
	QDialog::done ( r );
	close ( );
}


void Appearance::styleClicked ( int index )
{
	StyleListItem *sli = (StyleListItem *) m_style_list-> item ( index );
	m_style_settings-> setEnabled ( sli ? sli-> hasSettings ( ) : false );
	
	if ( m_sample && sli && sli-> style ( )) {
		int ci = m_color_list ? m_color_list-> currentItem ( ) : -1;
        
		m_sample-> setStyle2 ( sli-> style ( ), ci < 0 ? palette ( ) : ((ColorListItem *) m_color_list-> item ( ci ))-> palette ( ));
	}
		
	m_style_changed |= ( index != m_original_style );
}

void Appearance::styleSettingsClicked ( )
{
	StyleListItem *item = (StyleListItem *) m_style_list-> item ( m_style_list-> currentItem ( ));

	if ( item && item-> hasSettings ( )) {
		QDialog *d = new QDialog ( this, "SETTINGS-DLG", true );
		QVBoxLayout *vbox = new QVBoxLayout ( d, 3, 0 );

		QWidget *w = item-> settings ( d );

		if ( w ) {
			vbox-> addWidget ( w );

			d-> setCaption ( w-> caption ( ));

		   	d-> showMaximized ( );
			bool accepted = ( d-> exec ( ) == QDialog::Accepted );

			if ( item-> setSettings ( accepted ))
				m_style_changed = true;
		}
		delete d;
	}
}

#if 0
void Appearance::decoClicked ( int index )
{
	DecoListItem *dli = (DecoListItem *) m_deco_list-> item ( index );
	
	if ( m_sample ) {
		if ( dli && dli-> interface ( ))
			m_sample-> setDecoration ( dli-> interface ( ));		
		else
			m_sample-> setDecoration ( new DefaultWindowDecoration ( ));
		m_sample-> repaint ( );
	}
	m_deco_changed |= ( index != m_original_deco );
}
#endif

void Appearance::fontClicked ( const QFont &f )
{
	m_font_changed |= ( f != m_sample-> font ( ));	
	m_sample-> setFont ( f );
}

void Appearance::colorClicked ( int index )
{
	ColorListItem *item = (ColorListItem *) m_color_list-> item ( index );

	if ( item )
		m_sample-> setPalette ( item-> palette ( ));

	m_color_changed |= ( item-> palette ( ) != qApp-> palette ( ));
}


void Appearance::editSchemeClicked ( )
{
	ColorListItem *item = (ColorListItem *) m_color_list-> item ( m_color_list-> currentItem ( ));

	int cnt = 0;
	QString labels [QColorGroup::NColorRoles];
	QColor colors [QColorGroup::NColorRoles];
	
	for ( QColorGroup::ColorRole role = (QColorGroup::ColorRole) 0; role != QColorGroup::NColorRoles; ((int) role )++ ) {
		QColor col = item-> color ( role );
		
		if ( col. isValid ( )) {
			labels [cnt] = item-> label ( role );
			colors [cnt] = col;
		
			cnt++;
		}
	}

	EditScheme* editdlg = new EditScheme( cnt, labels, colors, this, "editScheme", true );
	editdlg-> showMaximized ( );
	if ( editdlg-> exec ( ) == QDialog::Accepted ) {
		ColorListItem *citem = (ColorListItem *) m_color_list-> item ( 0 );
		cnt = 0;
			
		for ( QColorGroup::ColorRole role = (QColorGroup::ColorRole) 0; role != QColorGroup::NColorRoles; ((int) role )++ ) {
			if ( item-> color ( role ). isValid ( )) {
				citem-> setColor ( role, colors [cnt] );
				cnt++;
			}
		}
		
		m_color_list-> setCurrentItem ( 0 );
		colorClicked ( 0 );
		
		m_color_changed = true;
	}
	delete editdlg;
}


void Appearance::saveSchemeClicked()
{
	ColorListItem *item = (ColorListItem *) m_color_list-> item ( m_color_list-> currentItem ( ));

	if ( !item )
		return;

	QDialog *d = new QDialog ( this, 0, true );
	d-> setCaption ( tr( "Save Scheme" ));
	QLineEdit *ed = new QLineEdit ( d );
	( new QVBoxLayout ( d, 3, 3 ))-> addWidget ( ed );
	ed-> setFocus ( );

	if ( d-> exec ( ) == QDialog::Accepted ) {
		QString schemename = ed-> text ( );
		QFile file ( QPEApplication::qpeDir() + "/etc/colors/" + schemename + ".scheme" );
		if ( !file. exists ( ))
		{ 
			QPalette p = item-> palette ( );
		
			Config config ( file.name(), Config::File );
			config. setGroup( "Colors" );

			item-> save ( config );
							
			config. write ( ); // need to flush the config info first
			
			m_color_list-> insertItem ( new ColorListItem ( schemename, config ));
		}
		else
		{
			QMessageBox::information ( this, tr( "Save scheme" ), tr( "Scheme does already exist." ));
		}
	}
	delete d;
}

void Appearance::deleteSchemeClicked()
{
	ColorListItem *item = (ColorListItem *) m_color_list-> item ( m_color_list-> currentItem ( ));

	if ( !item )
		return;

	if ( m_color_list-> currentItem ( ) > 0 )
	{
		if ( QMessageBox::warning ( this, tr( "Delete scheme" ), tr( "Do you really want to delete\n" ) + item-> text ( ) + "?",
		        tr( "Yes" ), tr( "No" ), 0, 0, 1 ) == 0 ) {
			QFile::remove ( QPEApplication::qpeDir ( ) + "/etc/colors/" + item-> text ( ) + ".scheme" );
			delete item;
		}
	}
	else
	{
		QMessageBox::information( this, tr( "Delete scheme" ), tr( "Unable to delete current scheme." ));
	}
}


void Appearance::addExcept ( )
{
	ExceptListItem *it = new ExceptListItem ( m_except, 0, tr( "<new>" ), true, true, true );
	m_except-> ensureItemVisible ( it );
	m_except-> setSelected ( it, true );
}

void Appearance::delExcept ( )
{
	if ( m_except-> selectedItem ( )) {
		m_except-> setFocus ( );
		delete m_except-> selectedItem ( );
	}
}

void Appearance::upExcept ( )
{
	ExceptListItem *it = (ExceptListItem *) m_except-> selectedItem ( );
	
	if ( it && it-> itemAbove ( ))
		it-> itemAbove ( )-> moveItem ( it );
}

void Appearance::downExcept ( )
{
	ExceptListItem *it = (ExceptListItem *) m_except-> selectedItem ( );
	
	if ( it && it-> itemBelow ( ))
		it-> moveItem ( it-> itemBelow ( ));
}

class ExEdit : public QLineEdit {
public:
	ExEdit ( ExceptListItem *item ) 
		: QLineEdit ( item-> listView ( )-> viewport ( ), "exedit" ), it ( item )
	{ 
		setFrame ( false );
		
		QRect r = it-> listView ( )-> itemRect ( it );
		
		int x = it-> listView ( )-> header ( )-> cellPos ( 3 ) - 1;
		int y = r. y ( );
		int w = it-> listView ( )-> viewport ( )-> width ( ) - x;
		int h = r. height ( ); // + 2;
		
		setText ( it-> pattern ( ));		
		setGeometry ( x, y, w, h );
		
		qDebug ( "ExEdit: [%s] at %d,%d %d,%d", it->text(2).latin1(),x,y,w,h);	
		
		m_out = true;
		
		show ( );
		setFocus ( );
		selectAll ( );
		end ( true );
	}
	
	virtual void focusOutEvent ( QFocusEvent * ) 
	{
		hide ( );
		if ( m_out )
			it-> setPattern ( text ( ));
		delete this;
	}
	
	virtual void keyPressEvent ( QKeyEvent *e )
	{
		if ( e-> key ( ) == Key_Return )
			it-> listView ( )-> setFocus ( );
		else if ( e-> key ( ) == Key_Escape ) {
			m_out = false;
			it-> listView ( )-> setFocus ( );
		}
		else
			QLineEdit::keyPressEvent ( e );
	}
	
private:
	ExceptListItem *it;
	bool m_out;
};

void Appearance::clickedExcept ( QListViewItem *item, const QPoint &, int c )
{
	if ( !item || c < 0 || c > 3 )
		return;

	ExceptListItem *it = (ExceptListItem *) item;

	if ( c == 0 )
		it-> setNoStyle ( !it-> noStyle ( ));
	else if ( c == 1 )
		it-> setNoFont ( !it-> noFont ( ));
	else if ( c == 2 )
		it-> setNoDeco ( !it-> noDeco ( ));
	else if ( c == 3 ) {
		m_except-> ensureItemVisible ( it );
		new ExEdit ( it );
	}
}


	

QWidget *Appearance::createLauncherTab ( QWidget *parent, Config & )
{
	Config cfg ( "zstyle" );
	cfg. setGroup ( "Launcher" );

	QWidget *tab = new QWidget( parent, "BgTab" );
	QVBoxLayout *vertLayout = new QVBoxLayout( tab, 3, 3 );
		
	QGridLayout* gridLayout = new QGridLayout ( vertLayout );
	gridLayout-> setColStretch ( 1, 10 );

	QLabel *label = 0;
	QHBoxLayout *hb = 0;

	if ( m_version < 3 ) {

	label = new QLabel( tr( "Type:" ), tab );
	gridLayout-> addWidget ( label, 0, 0 );
	m_bgtype = new QButtonGroup( tab, "buttongroup" );
	m_bgtype-> hide ( );
	m_bgtype-> setExclusive ( true );
	connect ( m_bgtype, SIGNAL( clicked ( int )), this, SLOT( bgTypeClicked ( int )));

	QRadioButton *rb;
	rb = new QRadioButton( tr( "Ruled" ), tab, "ruled" );
	m_bgtype-> insert ( rb, Ruled );
	gridLayout-> addWidget( rb, 0, 1 );

	hb = new QHBoxLayout ( );
	hb-> setSpacing ( 3 );

	rb = new QRadioButton( tr( "Solid color" ), tab, "solid" );
	m_bgtype-> insert ( rb, SolidColor ); 
	hb-> addWidget ( rb );
	hb-> addSpacing ( 10 );
	
	m_solidcolor = new OColorButton ( tab );
	connect ( m_solidcolor, SIGNAL( colorSelected ( const QColor & )), this, SLOT( bgColorClicked ( const QColor & )));
	hb-> addWidget ( m_solidcolor );
	hb-> addStretch ( 10 );

	gridLayout-> addLayout ( hb, 1, 1 );

	hb = new QHBoxLayout ( );
	hb-> setSpacing ( 3 );
	
	rb = new QRadioButton( tr( "Image" ), tab, "image" );
	m_bgtype-> insert ( rb, Image );
	hb-> addWidget( rb );
	hb-> addSpacing ( 10 );
	
	m_imagebrowse = new QPushButton ( tr( "Select..." ), tab );
	connect ( m_imagebrowse, SIGNAL( clicked ( )), this, SLOT( bgImageClicked ( )));
	hb-> addWidget ( m_imagebrowse );
	hb-> addStretch ( 10 );
	
	gridLayout-> addLayout ( hb, 2, 1 );

	}
	else {
		m_imagebrowse = 0;
		m_bgtype = 0;
		m_solidcolor = 0;
	}


	label = new QLabel( tr( "Text:" ), tab );
	gridLayout-> addWidget ( label, 3, 0 );
	
	hb = new QHBoxLayout ( );
	hb-> setSpacing ( 3 ); 
   
	m_iconcolor_use = new QCheckBox ( tr( "Custom color" ), tab );
	connect ( m_iconcolor_use, SIGNAL( toggled ( bool )), this, SLOT( customIconColorClicked ( bool )));
	hb-> addWidget ( m_iconcolor_use );
	hb-> addSpacing ( 10 );
	
	m_iconcolor = new OColorButton ( tab );
	connect ( m_iconcolor, SIGNAL( colorSelected ( const QColor & )), this, SLOT( iconColorClicked ( const QColor & )));
	hb-> addWidget ( m_iconcolor );
	hb-> addStretch ( 10 );
		
	gridLayout-> addLayout ( hb, 3, 1 );			

	hb = new QHBoxLayout ( );
	hb-> setSpacing ( 3 ); 
   
	m_iconfont_use = new QCheckBox ( tr( "Custom font" ), tab );
	connect ( m_iconfont_use, SIGNAL( toggled ( bool )), this, SLOT( customIconFontClicked ( bool )));
	hb-> addWidget ( m_iconfont_use );
	hb-> addSpacing ( 10 );
	
	m_iconfont_btn = new QPushButton ( tr( "Select..." ), tab );
	connect ( m_iconfont_btn, SIGNAL( clicked ( )), this, SLOT( iconFontClicked ( )));
	hb-> addWidget ( m_iconfont_btn );
	hb-> addStretch ( 10 );
		
	gridLayout-> addLayout ( hb, 4, 1 );			

	m_iconfont_dlg = new QDialog ( this, "iconfontdlg", true );
	m_iconfont_dlg-> setCaption ( tr( "Select font" ));
	hb = new QHBoxLayout ( m_iconfont_dlg );
	hb-> setSpacing ( 3 );
	hb-> setMargin ( 3 );
	m_iconfont_select = new OFontSelector ( true, m_iconfont_dlg, "fontsel" );	
	hb-> addWidget ( m_iconfont_select );
	
	connect ( m_iconfont_use, SIGNAL( toggled ( bool )), m_iconfont_btn, SLOT( setEnabled ( bool )));
	connect ( m_iconcolor_use, SIGNAL( toggled ( bool )), m_iconcolor, SLOT( setEnabled ( bool )));

	label = new QLabel( tr( "Icons:" ), tab );
	gridLayout-> addWidget ( label, 5, 0 );

	m_smallicons = new QCheckBox ( tr( "Small document icons" ), tab );
	gridLayout-> addWidget ( m_smallicons, 5, 1 );
	
	if ( m_version == 3 ) {
		label-> hide ( );
		m_smallicons-> hide ( );
	}
	
	vertLayout-> addStretch ( 10 );
	
	bool smallicons = ( cfg. readEntry ( "DocView", "Icon" ) == "List" );
	BackgroundType bgtype = Ruled;
	
	QString s = cfg. readEntry ( "BackgroundType" );
	if ( s == "SolidColor" )
		bgtype = SolidColor;
	else if ( s == "Image" ) // No tr
		bgtype = Image;
	else
		bgtype = Ruled;

	bool fontuse = false;
	QStringList f = cfg. readListEntry ( "Font", ',' );
	if ( f. count ( ) == 4 ) {
		fontuse = true;
	
		m_iconfont_select-> setSelectedFont ( QFont ( f [0], f [1]. toInt ( ), f [2]. toInt ( ), ( f [3]. toInt ( ))));
		m_sample2-> setViewFont ( m_iconfont_select-> selectedFont ( ));
	} 		
	m_iconfont_use-> setChecked ( fontuse );
	m_iconfont_btn-> setEnabled ( fontuse );
	
	m_smallicons-> setChecked ( smallicons );

	s = cfg. readEntry ( "TextColor" );
	bool coloruse = ( !s. isEmpty ( ) && QColor ( s ). isValid ( ));
	m_iconcolor-> setColor ( coloruse ?  QColor ( s ) : colorGroup ( ). color ( QColorGroup::Text ));
	m_iconcolor_use-> setChecked ( coloruse );
	m_iconcolor-> setEnabled ( coloruse );
	if ( coloruse )
		iconColorClicked ( m_iconcolor-> color ( ));

	if ( m_bgtype )
		m_bgtype-> setButton ( bgtype );
	if ( m_solidcolor )
		m_solidcolor-> setColor ( QColor ( cfg. readEntry ( "BackgroundColor", colorGroup ( ). color ( QColorGroup::Base ). name ( ))));
	m_bgimage = cfg. readEntry ( "BackgroundImage", "zstyle/logo" );
	if ( m_bgtype )
		bgTypeClicked ( bgtype );



	return tab;
}

void Appearance::customIconFontClicked ( bool b )
{
	m_sample2-> setViewFont ( b ? m_iconfont_select-> selectedFont ( ) : font ( ));
}

void Appearance::iconFontClicked ( )
{
	m_iconfont_dlg-> showMaximized ( );
	if ( m_iconfont_dlg-> exec ( ) == QDialog::Accepted )
		m_sample2-> setViewFont ( m_iconfont_select-> selectedFont ( ));
}

void Appearance::iconSizeClicked ( int s )
{
	m_sample2-> setViewMode ((ViewMode) s );
}


void Appearance::bgTypeClicked ( int t )
{
	QString s;

	if ( m_bgtype-> id ( m_bgtype-> selected ( )) != t )
		m_bgtype-> setButton ( t );
		
	m_solidcolor-> setEnabled ( t == SolidColor );
	m_imagebrowse-> setEnabled ( t == Image );

	if ( t == SolidColor )
		s = m_solidcolor-> color ( ). name ( );
	else if ( t == Image ) {
		s = Resource::findPixmap ( m_bgimage );
		if ( s. isEmpty ( ))
			s = m_bgimage;
	}
		
	m_sample2-> setBackgroundType ((BackgroundType) t, s );
}

void Appearance::bgColorClicked ( const QColor & )
{
	bgTypeClicked ( SolidColor );
}

void Appearance::customIconColorClicked ( bool b )
{
	iconColorClicked ( b ? m_iconcolor-> color ( ) : colorGroup ( ). color ( QColorGroup::Text ));
}

void Appearance::iconColorClicked ( const QColor &col )
{
	m_sample2-> setTextColor ( col );
}

void Appearance::bgImageClicked ( )
{
	// ### use OFileSelector here ###
	// this is just a quick c&p from the old appearance app

	MimeTypes types;
	QStringList list;
	list << "image/*";	
	types. insert ( "Images", list );
	
	QString file = OFileDialog::getOpenFileName ( 1, "/", QString::null, types );
	if ( !file. isEmpty ( )) {
		m_bgimage = DocLnk ( file ). file ( );
		bgTypeClicked ( Image );
	}
}


